using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace XELF.Effects.Noise {

	public class NoiseGame : Microsoft.Xna.Framework.Game {
		GraphicsDeviceManager graphics;
		SpriteBatch spriteBatch;
		
		public NoiseGame() {
			graphics = new GraphicsDeviceManager(this);
			Content.RootDirectory = "Content";
		}

		protected override void LoadContent() {
			spriteBatch = new SpriteBatch(GraphicsDevice);
			quad = new QuadFiller(GraphicsDevice);
			effect = Content.Load<Effect>("Noise");
		}
		Vector2 offset;
		Matrix[] transforms = new Matrix[5];
		float Frequency = 257 * 257;//243;
		Effect effect;
		QuadFiller quad;

		KeyboardState previousKeyState;
		KeyboardState currentKeyState;
		bool IsKeyPress(Keys key) {
			return currentKeyState.IsKeyDown(key) && previousKeyState.IsKeyDown(key);
		}

		enum Style {
			Noise1,
			Noise2,
			Noise3,
			Noise4,
			Noise5,
			Fire,
			Stripe,
			Atlas,
		}

		Style style = Style.Fire;
		Vector2 scale = new Vector2(2, 1);

		void HandleInput() {
			if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
				this.Exit();

			previousKeyState = currentKeyState;
			currentKeyState = Keyboard.GetState();
			if (IsKeyPress(Keys.F)) {
				style = Style.Fire;
				scale.X = 2; scale.Y = 1;
			}
			if (IsKeyPress(Keys.S)) {
				style = Style.Stripe;
				scale.X = 2; scale.Y = 1;
			}
			if (IsKeyPress(Keys.A)) {
				style = Style.Atlas;
				scale.X = 1; scale.Y = 1;
			}
			if (IsKeyPress(Keys.F1)) {
				style = Style.Noise1;
				scale.X = 1; scale.Y = 1;
			}
			if (IsKeyPress(Keys.F2)) {
				style = Style.Noise2;
				scale.X = 1; scale.Y = 1;
			}
			if (IsKeyPress(Keys.F3)) {
				style = Style.Noise3;
				scale.X = 1; scale.Y = 1;
			}
			if (IsKeyPress(Keys.F4)) {
				style = Style.Noise4;
				scale.X = 1; scale.Y = 1;
			}
			if (IsKeyPress(Keys.F5)) {
				style = Style.Noise5;
				scale.X = 1; scale.Y = 1;
			}

		}

		protected override void Update(GameTime gameTime) {
			HandleInput();

			offset.X += 0.008f;
			offset.Y += 0.03f;
			float a = GraphicsDevice.Viewport.AspectRatio * scale.X;
			float b = scale.Y;
			transforms[0] = Matrix.CreateScale(4 * a, 4 * b, 1) * Matrix.CreateTranslation(-offset.X, offset.Y, 0);
			transforms[1] = Matrix.CreateScale(8 * a, 8 * b, 1) * Matrix.CreateTranslation(offset.X * 3, offset.Y * 1.5f, 0);
			transforms[2] = Matrix.CreateScale(16 * a, 16 * b, 1) * Matrix.CreateTranslation(offset.X * 2, offset.Y * 0.5f, 0);
			transforms[3] = Matrix.CreateScale(32 * a, 32 * b, 1) * Matrix.CreateTranslation(offset.X * 1, offset.Y * 2.0f, 0);
			transforms[4] = Matrix.CreateScale(64 * a, 64 * b, 1) * Matrix.CreateTranslation(-offset.X * 1.5f, offset.Y * 1.6f, 0);
			effect.CurrentTechnique = effect.Techniques[style.ToString()];
			effect.Parameters["Frequency"].SetValue(Frequency);
			effect.Parameters["Transforms"].SetValue(transforms);

			base.Update(gameTime);
		}
		protected override void Draw(GameTime gameTime) {
			GraphicsDevice.Clear(Color.Black);
			quad.Fill(effect);
			base.Draw(gameTime);
		}
	}

	class QuadFiller : IDisposable {
		readonly GraphicsDevice graphicsDevice;
		readonly VertexDeclaration declaration;
		public QuadFiller(GraphicsDevice graphicsDevice) {
			this.graphicsDevice = graphicsDevice;
			declaration = new VertexDeclaration(graphicsDevice, VertexPositionTexture.VertexElements);
		}
		readonly VertexPositionTexture[] vertices = new VertexPositionTexture[] {
			new VertexPositionTexture(new Vector3(-1, +1, 0), new Vector2(0, 0)),
			new VertexPositionTexture(new Vector3(+1, +1, 0), new Vector2(1, 0)),
			new VertexPositionTexture(new Vector3(-1, -1, 0), new Vector2(0, 1)),
			new VertexPositionTexture(new Vector3(+1, -1, 0), new Vector2(1, 1)),
		};
		public void Dispose() {
			declaration.Dispose();
		}
		public void Fill(Effect effect) {
			effect.Begin();
			foreach (var pass in effect.CurrentTechnique.Passes) {
				pass.Begin();
				Fill();
				pass.End();
			}
			effect.End();
		}
		public void Fill() {
			graphicsDevice.VertexDeclaration = declaration;
			graphicsDevice.DrawUserPrimitives(PrimitiveType.TriangleStrip, vertices, 0, 2);
		}
		public void Fill(Effect effect, float x0, float y0, float x1, float y1) {
			effect.Begin(SaveStateMode.SaveState);
			foreach (var pass in effect.CurrentTechnique.Passes) {
				pass.Begin();
				Fill(x0, y0, x1, y1);
				pass.End();
			}
			effect.End();
		}
		public void Fill(float x0, float y0, float x1, float y1) {
			VertexPositionTexture[] vertices = new VertexPositionTexture[] {
				new VertexPositionTexture(new Vector3(x0, y1, 0), new Vector2(0, 0)),
				new VertexPositionTexture(new Vector3(x1, y1, 0), new Vector2(1, 0)),
				new VertexPositionTexture(new Vector3(x0, y0, 0), new Vector2(0, 1)),
				new VertexPositionTexture(new Vector3(x1, y0, 0), new Vector2(1, 1)),
			};
			graphicsDevice.VertexDeclaration = declaration;
			graphicsDevice.DrawUserPrimitives(PrimitiveType.TriangleStrip, vertices, 0, 2);
		}
	}

}
